import { delegateEvents, spread, mergeProps, insert, createComponent, setAttribute, effect, className, Dynamic, use, template } from 'solid-js/web';
import { List } from '@solid-primitives/list';
import { createDeepSignal } from '@solid-primitives/resource';
import clsx from 'clsx';
import { getHighlighter, codeToHast } from 'shiki';
import { createSignal, createMemo, useTransition, createResource, createEffect, Show, Index, splitProps, mergeProps as mergeProps$1, onCleanup } from 'solid-js';
import { extend, colord } from 'colord';
import mixPlugin from 'colord/plugins/mix';

// src/index.tsx

// src/index.module.css
var src_default = {};
extend([mixPlugin]);
function calculateContrastingColor(hexColor) {
  const backgroundColor = colord(hexColor);
  let selectionBackgroundColor;
  if (backgroundColor.isLight()) {
    selectionBackgroundColor = backgroundColor.darken(0.1);
  } else {
    selectionBackgroundColor = backgroundColor.lighten(0.15);
  }
  return selectionBackgroundColor.toRgbString();
}

// src/utils/conditionals.ts
function when(accessor, callback, fallback) {
  const value = typeof accessor === "function" ? accessor() : accessor;
  return value ? callback(value) : fallback ? fallback() : void 0;
}
function whenever(accessor, callback, fallback) {
  return () => when(accessor, callback, fallback);
}
function processProps(props, defaults, split) {
  return splitProps(defaultProps(props, defaults), split || []);
}
function defaultProps(props, defaults) {
  return mergeProps$1(defaults, props);
}

// src/index.tsx
var _tmpl$ = /* @__PURE__ */ template(`<div><div><code></code><textarea inputmode=none autocomplete=off>`);
var _tmpl$2 = /* @__PURE__ */ template(`<code aria-hidden>`);
var calculateMaxCharCount = (source) => {
  let maximumLineSize = -Infinity;
  source.split("\n").forEach((line) => {
    if (line.length > maximumLineSize) {
      maximumLineSize = line.length;
    }
  });
  return maximumLineSize;
};
function ShikiTextarea(props) {
  const [config, rest] = processProps(props, {
    lang: "tsx",
    theme: "min-light"
  }, ["class", "defaultValue", "lang", "onInput", "value", "style", "theme"]);
  const [source, setSource] = createSignal(config.defaultValue || config.value);
  const [characterDimensions, setCharacterDimensions] = createSignal({
    width: 0,
    height: 0
  });
  const maxCharCount = createMemo(() => calculateMaxCharCount(source()));
  const lineCount = createMemo(() => source().split("\n").length);
  const [, startTransition] = useTransition();
  const [themeStyles] = createResource(() => [config.theme, config.lang], ([theme, lang]) => getHighlighter({
    themes: [theme],
    langs: [lang]
  }).then((highlighter) => highlighter.getTheme(theme)).then((theme2) => ({
    "--theme-selection-color": calculateContrastingColor(theme2.bg),
    "--theme-background-color": theme2.bg,
    "--theme-foreground-color": theme2.fg
  })));
  createEffect(() => setSource(config.value));
  const [hast] = createResource(() => [source(), config.theme, config.lang], ([source2, theme, lang]) => source2 ? codeToHast(source2, {
    lang,
    theme
  }).then(({
    children
  }) => children[0].children[0]) : {
    children: void 0
  }, {
    storage: createDeepSignal
  });
  const hastNodes = () => (hast() || hast.latest)?.children;
  return (() => {
    var _el$ = _tmpl$(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.nextSibling;
    spread(_el$, mergeProps({
      get ["class"]() {
        return clsx(src_default.editor, config.class);
      },
      get style() {
        return {
          ...themeStyles(),
          ...config.style,
          "--line-count": lineCount(),
          "--max-char-count": maxCharCount(),
          "--char-width": characterDimensions().width,
          "--char-height": characterDimensions().height
        };
      }
    }, rest), false, true);
    insert(_el$3, createComponent(List, {
      get each() {
        return hastNodes();
      },
      children: (line) => createComponent(HastNode, {
        get node() {
          return line();
        }
      })
    }));
    _el$4.$$input = ({
      currentTarget: {
        value
      }
    }) => {
      startTransition(() => {
        setSource(value);
        config.onInput?.(value);
      });
    };
    setAttribute(_el$4, "spellcheck", false);
    insert(_el$2, createComponent(CharacterDimensions, {
      onResize: setCharacterDimensions
    }), null);
    effect((_p$) => {
      var _v$ = src_default.container, _v$2 = src_default.textarea;
      _v$ !== _p$.e && className(_el$2, _p$.e = _v$);
      _v$2 !== _p$.t && className(_el$4, _p$.t = _v$2);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    effect(() => _el$4.value = config.value);
    return _el$;
  })();
}
function HastNode(props) {
  return createComponent(Show, {
    get when() {
      return props.node.type !== "text" && props.node;
    },
    get fallback() {
      return props.node.value;
    },
    children: (node) => createComponent(Dynamic, mergeProps({
      get component() {
        return node().tagName || "div";
      }
    }, () => node().properties, {
      get children() {
        return createComponent(Index, {
          get each() {
            return node().children;
          },
          children: (child) => createComponent(HastNode, {
            get node() {
              return child();
            }
          })
        });
      }
    }))
  });
}
function CharacterDimensions(props) {
  const [character, setCharacter] = createSignal(null);
  createEffect(whenever(character, (character2) => {
    const resizeObserver = new ResizeObserver(() => {
      const {
        width,
        height
      } = character2.getBoundingClientRect();
      props.onResize({
        width,
        height
      });
    });
    resizeObserver.observe(character2);
    onCleanup(() => resizeObserver.disconnect());
  }));
  return (() => {
    var _el$5 = _tmpl$2();
    use(setCharacter, _el$5);
    _el$5.innerText = "m";
    effect(() => className(_el$5, src_default.character));
    return _el$5;
  })();
}
delegateEvents(["input"]);

export { ShikiTextarea };
